# Filtering the AST #
In order to view the nodes you are interested in we have created a Filter Configuration Language, let's call it FCL. FCL more or less allows you to filter your AST, into something we call a filtered tree. But this is just a small technicality so lets start the quest to understand this language.

* Filtering the AST
    - The Structure
    - The Configuration block
    - The Filter block
         - When
         - Subtree
         - Show
         - Style
    - A quick way to filter
    - Hierarchy of filters and AST-nodes
    - Big Example


The following sections will explain how to use it. 
## Basic structure ##

The structure of the FCL is quite basic, it contains a ```configs``` block and a number of ```filter``` blocks. These will be explained later on in more detail.
But for now lets look at the code example below, which  show this structure. Yey.

```
configs{
  use = f1, f2;
}
filter f1{
  ...
}
filter f2{
...
}
```

The ``` configs ``` block, in the code above, determine which filters that should be enabled, in this case filter ``` f2 ``` and ``` f1 ```. Where as the ``` filter ``` blocks determine which AST nodes that should be present in your filtered tree. 

This is basically it for the structure, so lets dive into these blocks to see what we can use them for.

## The Configuration block ##

Ok, as mentioned before the ``` configs ``` block determine which filters that should be enabled when building your filtered tree. So by writing the ``` use ``` configuration you can enable any filter you want, if you have written any. 

```
configs{
  use = f1, f2; //Yey two filters enabled :D
}
filter f1{
  ...
}
filter f2{
...
}
```
On top of this you can add other configurations to this block, which are listed below. 

The following configs are currently supported:
```
use = [FilterName]+; // Enable 1 or more filters
dynamic-values = [on, off] //Will dynamically compute the attributes of a node when clicked in the graph, if on. Standard value is off
NTA-cached = [on, off] //Will determine if cached NTAs should be displayed. Standard value is on
NTA-depth == [Integer]; // Will determine the max depth of the displayed NTAs, if they are recursive they will stop at the max depth.  
NTA-computed = [on,off]; // Will determine if NTAs created by direct user calls should be displayed. 
```
The last two configurations above make no sense at the moment, i.e. they will be explained later on, so lets instead continue with the  ``` filter  ``` blocks. 
## The Filter block ##

So this is where the magic happens. In the ``` filter ``` blocks you can list which AST node types that should be included from you AST in the filtered tree. This is done by annotating your AST node types with ``` : ```. Se below for a example of the syntax.

```
configs{
  use = f1, f2;
}
filter f1{
  :SomeNode;
  :AnotherNode{} //Also a valid syntax
}
filter f2{
  :YetAnotherNode{ //Also a valid syntax
    when{...}
    style{...}
    show{...}
  }
  :SomeSuperNode; //All my class children will be visible YEY
}
```

Ok, filter ``` f1 ``` is quite obvious what it does, it will include all the AST-nodes with the types ``` SomeNode ``` and ``` AnotherNode ```. So lets take a look at the second filter ``` f2 ```. This will include the AST nodes with the type ``` YetAnotherNode ```, but what does the ```when```- , ```style```- and ```show``` blocks do? Continue reading to find out. 

But wait! What does ``` SomeSuperNode ``` do? Okej, lets do this. ``` SomeSuperNode ``` will force all class children, i.e the AST node types, that inherit ``` SomeSuperNode ```  to be included in the filtered tree.

Note: All names from your AST should be annotated with ``` : ``` (nodes types, attributes, tokens).

### When

Ohh good you continued to read, good for you! 
So this sub-block is the most "advanced" thing in the FCL. In this block you can list conditions that your AST nodes must fulfill to be included in the filtered tree. The ``` when ``` block is only applied for the AST nodes with the same type, or sub classes of the type, in which the ``` when ``` block is defined. So the ``` when ``` block in the code below is only applied to AST nodes with the type ``` YetAnotherNode ``` (and sub classes of  ``` YetAnotherNode  ```, see later sections for more information about this).
```
filter f2{
  // a, b, d, e are attributes, with values that corresponds to the expression type.
  :YetAnotherNode {
    when{
      5 > :a;
      :b == "Hello Filter";
      :c in [1,2,5,61];
      not d:
      :e;
      child of :SomeNode;
      not parent of :AnotherNode;
    }
  }
}
```

Lets check out which value types and expressions that are supported in FCL, to get a better grip of what the code above does.

The supported types for FCL, are the following:

- A Boolean: true, false.
- Integer: any Java int.
- String: any Java String.
- An array of Integers or Strings: Like this [1,1,2].
- An attribute name: annotated with ``` : ```
- A AST node type name: annotated with ``` : ```
    

Here is a list of the different expressions in FCl:

```
// x and y are attribute names.

//Comparators single simple attribute expressions
:x [==, !=, <, >, <=, >=] <Integer>;
:x [==, !=] <String>;
:x [==, !=] <Boolean>;

//Comparators for Integer attributes
:x [==, !=, <, >, <=, >=] :y;

//Comparators for String and Boolean attributes
:x [==, !=] :y;

//Comparators for single Boolean attributes
not :x;       // :x == false;
:x;           // :x == true;

//Array expression
:x not in Array(<String> | <Integer>);
:x in Array(<String> | <Integer>);

//ASTNodeName is a simple name of a AST-node type
child of :ASTNodeType;
not child of :ASTNodeType;
parent of :ASTNodeType;
not parent of :ASTNodeType;
```

Note: it is important that attributes value type correspond with the correct type in the expression they are used in.

Alrighty then lets move onward with the quest!

### Subtree ###

For HUGE ASTs it might be good to filter a whole subtree of a node, this cool feature can be done with the ``` subtree ``` block. Lets look at an exampel, the one below :).
```
filter f2{
  :YetAnotherNode{
    when{
      ...
    }
    subtree{
      :getID == "Hello";
      child of :SomeNode;
      when shown;
    }
  } 
}
```

The ``` subtree ``` block works like the when block, and supports the same conditions. Yey conditions.

So if the conditions in a ``` subtree ``` block are fulfilled the subtree of that node will be included in the filtered tree. So node of the type ``` YetAnotherNode ``` will have subtree if it has an attribute with the name ``` :getID ``` with the value "Hello" and if it is a child of a node of the type ``` SomeNode ```. 

But wait! What does ``` when shown; ``` do? Well it basically says that subtree of ``` YetAnotherNode ``` should be included if it also fulfills the ``` when ``` blocks states. So when nodes of the type ``` YetAnotherNode ``` is included, i.e fulfills the ``` when ``` block, and fulfills the other conditions in the ``` subtree ``` the node will have a subtree. 

Don't fret! Empty or non-existing  ``` subtree ``` blocks are always ``` true ```.

Phew that might have been a bit much, but hopefully you now understand the conditions of the ``` when ``` and ``` subtree ``` blocks. 

So lets continue with something simpler, like for example displaying attributes directly in the filtered tree.

### Show ###

In the show-block you can define things you would like to see directly in the filtered tree, in the DrASTGUI they will be displayed directly in the graph. This is done by simply listing them in the ``` show ``` block. Se the code below for syntax.

```
filter f2{
  :YetAnotherNode{
    show{
      :print;
      :someValue;
      :aReferenceAttribute;
    }
  } 
}
```
The attributes will be displayed in the following way.

If the attribute is a primitive value, it will be displayed as "name : value", but if it is a reference to another node, the reference will be pointed out in the graph as an edge. 

We also have added support for NTA:s, and you can use them as the any other attribute in show.
NTA:s will be represented as nodes in the filtered tree, they will also be pointed out in the graph as an edge. 

Ok, remember the configuration that didn't make any sense in the configuration block? Let me try to explain them now.

The ``` NTA-depth = 10 ``` configuration will determine the max depth of NTA-chains, i.e. NTA:s that create other NTA:s. A small example of this is: if a node of type ``` A  ``` creates a node of type ``` B ``` and the ``` B  ``` node, for some reason creates a node of type ``` A ``` which in turn create a new node of type ``` B ```.... this will never end, except when you run out of memory. Hurray. So this configuration will simply stop these chains at the NTA-depth value, in the case below it will stop at the depth 10.

Now what about the ``` NTA-computed ``` ? So this will determine if any NTA:s created by the user, in the  user interface component, should be visible or not.

```
configs{
  use = f1;
  NTA-depth = 10;
  NTA-computed = on;
}
filter f1{
  :YetAnotherNode{
    show{
      :getNum; /*NTA*/
    }
  }
}
```

Yey visual things are fun so lets continue with it, namely the ``` style ``` block.

### Style ###

So you want to style your filtered tree I see, feel free to use the ``` style ``` block you see.

With the ``` style ``` block you can define styles for your AST nodes in the filtered tree. The style-block are connected to the type in which it is defined, like the ``` when ``` block. 

```
configs{
  use = f1, f2;
}
filter f1{
  :SomeNode;
}
filter f2{
  :YetAnotherNode{
    when{}
    style{
      node-color = #FF0000; //Ohh so red
      node-shape = "circle"; //Yey I'm gonna be round
    }
  } 
}
```

Note: The ``` style ``` block will only be applied if the filter in which the styles are defined is enabled in the configs.

The JastAddAd UI currently supports the following styles:
```
node-color = #<6 digit color code>;
node-shape = "circle" | "rounded_rectangle" | "rectangle";
border-style = "dashed" | "line";
```

Ok, now the main quest of understanding the language is completed. But let's continue to dive a bit deeper.

## A quick way to filter ##

So writing filter for all type can be a bit dull, so we have added support for inheritance. This will enable you to apply blocks for a super-class and all the types that inherit from that will be affected. 

```
configs{
  use = f1;
}
filter f1{
  :SuperNodeType{
    when{ ... } //Some filter
    style{ ... } //Some style
    show{ ... } //Some show
  }
  //These nodes inherit SuperNodeType
  :SomeNode; //I have the same conditions, style and shown attributes as my parent
  :YetAnotherNode; //I have the same conditions, style and shown attributes as my parent
  
}
```

Ok, what about inheritance ambiguities? This will be addressed below. 

## Hierarchy of filters and AST-nodes ##

Lets see how multiple filters that define the same AST-node types are handled, as well as the scary super-classes.

The main thing for both multiple filters and inheritance is that the ``` when ``` block works as an union of all conditions, i.e. they will be collected into a single set. So be careful when writing ``` when ``` blocks for super-classes or in multiple filters so that the conditions do not contradict themselves. 
For example ``` :x == 1; ``` and ``` :x == 2; ``` for the same AST node type can never be fulfilled.

How about ``` style ```? For inheritance they can be overridden by defining your own style value. But for multiple filters only one style is applied, and this will be the first style in the file. Se the example below. 

```
filter f1{
  :YetAnotherNode{
    style{ 
       node-color = #00FF00; //Color style one, this style will be applied
    } 
  }
}
filter f2{
  :YetAnotherNode{
    style{ 
       node-shape = "circle"; //Shape style one, this style will be applied.
       node-color = #FF00FF; //Color style two, this style is ignored 
    } 
  }
}
```

The last block ``` show ``` will just have a union of all the attributes from the multiple filters and super-classes.

You are now done with the quest and the tutorial, time to start filter and rejoice. Yey!
   
## Big Example ##

Here is a slightly bigger example that you should be able to understand after reading the above:
```
configs{
  use = f1, f2;
}
filter f1{
  :ASTNode{
    when{
      :x == 9;
    }
    style{
      node-color = #0000FF;
    }
  }
}
filter f2{
  :IdDecl;
  :Mul{
    when{
      y == 5;
      z in ["foo", "bar"];
    }
    style{ 
      node-color = #00ff00;
    }
  }
  :BinExpr {
    style{ 
      node-color = #ff0000;
      border-style = "dashed";
    }
  }
}
```